/* Part of the TSO unit test framework */

#ifndef TSO_POST_INCLUDE
#define TSO_POST_INCLUDE

struct ethtool_modinfo;
#undef CONFIG_BQL
#undef CONFIG_NETPOLL_TRAP

/* Horrible kluge to enable VLAN tag insertion without requiring kcompat */
#define EFX_USE_FAKE_VLAN_TX_ACCEL 1

#define warn_slowpath_null	my_warn_slowpath_null

/* We need to make this substitution before including <linux/skbuff.h> */
#include <linux/dma-mapping.h>
#undef dma_map_page
#define dma_map_page           my_dma_map_page
extern dma_addr_t my_dma_map_page(struct device *hwdev, struct page *page, size_t offset, size_t size, int direction);

/* Need this before including kernel_compat.h */
#undef page_address
#define page_address           my_page_address
struct page;
extern void *my_page_address(struct page *page);

#include "net_driver.h"
#include "nic.h"
#include "tx_common.h"

/* Avoid having to mock tracing */
#undef CONFIG_SFC_TRACING

#ifndef TCP_MAX_GSO_SEGS
#define TCP_MAX_GSO_SEGS 50
#endif

/* Misc */
extern unsigned int efx_debug_level;

#define EFX_PTR_ALIGN_FWD(i, align) \
  (void*)((((unsigned long)i)+(align)-1u) & ~((align)-1u))

#define MY_ASSERT(a) do { \
   if (!(a)) my_fail("ASSERT ERROR: %s at %s:%d\n",#a,__FUNCTION__,__LINE__); \
 } while(0)

#define MY_ASSERT_EQUAL(a,b) do { \
 if ((a)!=(b)) my_fail("ASSERT ERROR: %s != %s (%d != %d) at %s:%d\n", \
                         #a,#b,a,b,__FUNCTION__,__LINE__); \
 } while(0)


/* Functions from user land */
#include "tso_shared.h"

#define kmalloc(size, flags)   my_malloc(size)
#define kzalloc(size, flags)   my_calloc(1, size)
#define kcalloc(nmemb, size, flags) my_calloc(nmemb, size)
#define kfree(ptr)             my_free((void *)(ptr))
#define printk                 my_fail
#undef netdev_err
#define netdev_err(dev, format, ...) my_fail(format, ##__VA_ARGS__)
#undef netdev_printk
#define netdev_printk(level, dev, format, ...) \
                               my_netdev_printk(level, format, ##__VA_ARGS__)
#undef memcmp
#define memcmp(s1, s2, n)	my_memcmp(s1, s2, n)
#undef memcpy
#define memcpy(dest, src, n)	my_memcpy(dest, src, n)

/* wrapped functions */

#undef dma_alloc_coherent
#define dma_alloc_coherent     my_dma_alloc_coherent
#undef dma_free_coherent
#define dma_free_coherent      my_dma_free_coherent
#undef dma_unmap_page
#define dma_unmap_page         my_dma_unmap_page
#undef dma_map_single
#define dma_map_single         my_dma_map_single
#undef dma_unmap_single
#define dma_unmap_single       my_dma_unmap_single
#define dma_mapping_error      my_dma_mapping_error
#define dev_kfree_skb_any      my_dev_kfree_skb_any
#if !defined(EFX_USE_KCOMPAT) || !defined(EFX_NEED_CONSUME_SKB_ANY)
#define dev_consume_skb_any    my_dev_consume_skb_any
#endif
#define netif_tx_stop_queue    my_netif_tx_stop_queue
#define netif_tx_wake_queue    my_netif_tx_wake_queue
#define skb_copy_bits          my_skb_copy_bits
#define skb_cow_head         my_skb_cow_head
#define pskb_expand_head my_pskb_expand_head
#define jiffies                my_jiffies
#undef memcpy_toio
#define memcpy_toio	       my_memcpy_toio
#define skb_gso_segment        my_skb_gso_segment
#define efx_ptp_channel        my_efx_ptp_channel
#define efx_ptp_use_mac_tx_timestamps	my_efx_ptp_use_mac_tx_timestamps
#define page_frag_free         my_page_frag_free

extern void *my_dma_alloc_coherent(struct device *hwdev, size_t size, dma_addr_t *dma_handle);
extern void my_dma_free_coherent(struct device *hwdev, size_t size, void *vaddr, dma_addr_t dma_handle);
extern void my_dma_unmap_page(struct device *hwdev, dma_addr_t dma_addr, size_t size, int direction);
extern dma_addr_t my_dma_map_single(struct device *hwdev, void *ptr, size_t size, int direction);
extern void my_dma_unmap_single(struct device *hwdev, dma_addr_t dma_addr, size_t size, int direction);
extern int my_dma_mapping_error(struct device *hwdev, dma_addr_t dma_addr);
extern int my_dev_kfree_skb_any(struct sk_buff *skb);
#if !defined(EFX_USE_KCOMPAT) || !defined(EFX_NEED_CONSUME_SKB_ANY)
extern int my_dev_consume_skb_any(struct sk_buff *skb);
#endif
static inline void my_netif_tx_stop_queue(struct netdev_queue *dev_queue) {}
static inline void my_netif_tx_wake_queue(struct netdev_queue *dev_queue) {}
extern int my_skb_copy_bits(const struct sk_buff *skb, int offset,
			    void *to, int len);
extern int my_skb_cow_head(struct sk_buff *skb, unsigned int headroom);
extern int pskb_expand_head(struct sk_buff *skb,
			    int nhead, int ntail,
			    gfp_t gfp_mask);
extern void my_warn_slowpath_null(const char *file, int line);
extern void my_memcpy_toio(volatile void *to, const void *from,
			   unsigned int len);
extern void my_page_frag_free(void *p);

extern unsigned long my_jiffies;
static inline struct sk_buff *my_skb_gso_segment(struct sk_buff *skb,
						 int features)
{
	return NULL;
}
static inline struct efx_channel *efx_ptp_channel(struct efx_nic *efx)
{
	return NULL;
}
static inline bool efx_ptp_use_mac_tx_timestamps(struct efx_nic *efx)
{
	return false;
}

#define efx_ptp_is_ptp_tx(efx, skb) 0
#define efx_ptp_tx             my_efx_ptp_tx
#define efx_schedule_reset     my_efx_schedule_reset
#define efx_nic_alloc_buffer   my_efx_nic_alloc_buffer
#define efx_nic_free_buffer    my_efx_nic_free_buffer
#define efx_nic_probe_tx       my_efx_nic_probe_tx
#define efx_nic_remove_tx      my_efx_nic_remove_tx
#define efx_nic_init_tx        my_efx_nic_init_tx
#define efx_nic_fini_tx        my_efx_nic_fini_tx

static inline netdev_tx_t my_efx_ptp_tx(struct efx_nic *efx, struct sk_buff *skb)
{
	my_fail("efx_ptp_tx called\n");
	return 0;
}
extern void my_efx_schedule_reset(struct efx_nic *efx, enum reset_type type);
extern int my_efx_nic_alloc_buffer(struct efx_nic *efx, struct efx_buffer *buf,
				   unsigned int len, gfp_t flags);
extern void my_efx_nic_free_buffer(struct efx_nic *efx, struct efx_buffer *buf);
static inline int my_efx_nic_probe_tx(struct efx_tx_queue *tx_queue)
{ return 0; }
static inline void my_efx_nic_remove_tx(struct efx_tx_queue *tx_queue) {}
static inline int my_efx_nic_init_tx(struct efx_tx_queue *tx_queue) { return 0; }
static inline void my_efx_nic_fini_tx(struct efx_tx_queue *tx_queue) {}

#endif

