#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <assert.h>
#include <time.h>
#include "tso_shared.h"

unsigned int efx_debug_level = 10;

void vfail(const char *fmt, va_list ap)
{
	vprintf(fmt, ap);

	assert(0);
}

void fail(const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	vfail(fmt, ap);
	va_end(ap);
}

void netdev_vprintk(const char *level, const char *fmt, va_list ap)
{
	/* KERN_DEBUG can be ignored */
	if (strcmp(level, "<7>") == 0)
		return;

	vprintf(fmt, ap);

	assert(0);
}

void netdev_printk(const char *level, const char *fmt, ...)
{
	va_list ap;
	va_start(ap, fmt);
	netdev_vprintk(level, fmt, ap);
	va_end(ap);
}

int main(int argc, char **argv)
{
	int rc;

	functions.user_calloc = calloc;
	functions.user_malloc = malloc;
	functions.user_free = free;
	functions.user_puts = puts;
	functions.user_vprintf = vprintf;
	functions.user_vfail = vfail;
	functions.user_netdev_vprintk = netdev_vprintk;
	functions.user_memcmp = memcmp;
	functions.user_memcpy = memcpy;

	if (argc > 1 && strcmp(argv[1], "--verbose") == 0) {
		verbose = 1;
		++argv; --argc;
	}

	if (argc < 2 || argc > 4) {
		fprintf(stderr,
			"Usage: %s [--verbose] nic-revision [iterations] [seed]\n",
			argv[0]);
		return 2;
	}

	if (argc > 3)
		seed = strtoul(argv[3], NULL, 10);
	else
		seed = (unsigned int)time(NULL);

	if (argc > 2)
		iters = strtoul(argv[2], NULL, 10);

	nic_revision = strtoul(argv[1], NULL, 10);

	printf("seed=%lu\n", seed);

	rc = do_test();

	if (rc == 0)
		printf("Test passed\n");
	else
		printf("Test failed rc=%d\n", rc);

	return rc;
}
