/* Part of the TSO unit test framework */

#ifndef TSO_TEST_FRAMEWORK
#define TSO_TEST_FRAMEWORK

#include <linux/pci.h>
#include <linux/tcp.h>
#include <linux/ip.h>
#include <linux/in.h>
#include <linux/if_ether.h>
#include <linux/if_vlan.h>
#include <net/ipv6.h>
#include "efx.h"

#define IP_HLEN 20
#define TCP_HLEN 20

#define BUF_HEADER 1
#define BUF_DATA 0

/* mmmm global structures */
struct tso_test_state {
	char *data_base, *data_to_send, *data_to_check;
	void *data_base_free, *skb_head_free;
	int data_left;
	char *hdr_copy;       /* copy of the headers to check against */
	int tcp_paylen_left;  /* TCP payload to consume */
	int fin, psh, cwr;    /* flags */
	int gso_size;

	struct {
		void (*hex_dump) (const uint8_t * data, unsigned int len);
		int (*check_buffer) (struct efx_tx_buffer * txbuf,
				     const char *expect, int header);
	} ops;
};

extern int check_tx_queue(const struct efx_tx_queue *tx_queue,
			  struct tso_test_state *ts);

static inline void *dma_addr_to_ptr(dma_addr_t dma_addr)
{
	return (void *) (uintptr_t) dma_addr;
}

static inline dma_addr_t ptr_to_dma_addr(void *ptr)
{
	return (dma_addr_t) (uintptr_t) ptr;
}

#endif

/*
 * Local variables:
 *  c-basic-offset: 8
 *  c-indent-level: 8
 *  tab-width: 8
 *  indent-tabs-mode: 1
 * End:
 */
