#!/bin/sh
# This script is meant to compose the release notes from a number of sections
# (in order to avoid duplication)
set -eu

me=$(basename "$0")
bin=$(cd "$(dirname "$0")" && /bin/pwd)
export PATH="$bin:$PATH"

err  () { echo 1>&2 "$*";   }
log  () { err "$me: $*";     }
fail () { log "$*"; exit 1; }
try  () { "$@" || fail "'$*' failed"; }
try_quiet  () { "$@" 2>&1 > /dev/null || fail "'$*' failed"; }

cd $bin
# Note that the output is not checked in (so that people are not tempted to
# edit it; as then we could lose the edits)
mkdir -p tmp

if [ "$#" -eq 1 ]; then
    VERSION=$1
else
    TOP=$(git rev-parse --show-toplevel 2> /dev/null)
    VERSION=$(make -s -C $TOP version | sed 's/[_-]/./g')
fi

# expect sections named as section_$name.txt, and header_$target.txt
concat() {
    local target=$1; shift
    local sections=$*
    local output="tmp/$target.txt"
    cat "header_$target.txt" > $output
    for section in $sections; do
	cat section_divider.txt >> $output
	cat "section_$section.txt" >> $output
    done
    sed -i "s/__version__/$VERSION/" $output
    echo "Wrote: $bin/$output"
}

commonparts="
distro_support
distro_policy
support
changelog_net_driver
removed_features
partitioning_sriov_support
known_platform_issues
known_sfc_issues
copyright_banner"

concat linux_net_srpm_SF-103848-LS  \
       supported_hardware \
       srpm_build_install \
       $commonparts

concat linux_net_dkms_SF-104979-LS  \
       driver_updates \
       documentation \
       supported_hardware \
       dkms_build_install \
       $commonparts

concat linux_net_kmp_SF-121791-LS  \
       supported_hardware \
       kmp_install \
       $commonparts

concat linux_net_kmp_srpm_SF-121792-LS  \
       supported_hardware \
       kmp_srpm_build_install \
       $commonparts

# And warn the user about what version strings are present.
echo '(note if this has only packaging changes after the tag use "export USE_BASE_DRIVER_VERSION=1")'
echo "Used version string $VERSION"
