#!/bin/sh
# Create the required directory structure for a Dell release

me=$(basename "$0")
bin=$(cd "$(dirname "$0")" && /bin/pwd)
export PATH="$bin:$PATH"

err  () { echo 1>&2 "$*";   }
log  () { err "$me: $*"; }
fail () { log "$*"; exit 1; }
try  () { "$@" || fail "'$*' failed"; }
try_quiet  () { "$@" 2>&1 > /dev/null || fail "'$*' failed"; }

set -eu

usage() {
    fail "$0: [planet9|neptune] <release_name>"
}

[ $# = 2 ] || usage

PARTS="kmod_kmp_bin kmod_kmp_src kmod_kmp_debuginfo"
case "$1" in
    planet9)
	DISTROS="RHEL7.6 RHEL8.0 SLES15SP1"
	DOXBOXPARTS="BIN=>SF-121791-LS  SRC=>SF-121792-LS DBGINFO=>SF-121793-LS"
	;;
    neptune)
	DISTROS="RHEL7.7 RHEL8.1 SLES15SP1"
	DOXBOXPARTS="BIN=>SF-122714-LS  SRC=>SF-122715-LS DBGINFO=>SF-122716-LS"
	;;
    *)
	usage
	;;
esac  
REL=$2

for part in $PARTS; do
  for distro in $DISTROS; do
    mkdir -p "${REL}/${part}/${distro}"
  done
done

tree "${REL}"

echo "${DOXBOXPARTS}"
echo 
echo OK
