#!/bin/sh
# Zip up a directory for dell

me=$(basename "$0")
bin=$(cd "$(dirname "$0")" && /bin/pwd)
export PATH="$bin:$PATH"

err  () { echo 1>&2 "$*";   }
log  () { err "$me: $*"; }
fail () { log "$*"; exit 1; }
try  () { "$@" || fail "'$*' failed"; }
try_quiet  () { "$@" 2>&1 > /dev/null || fail "'$*' failed"; }

set -eu

usage() {
    fail "$0: [planet9|neptune] <release_name>"
}

checkdistros() {
    local part=$1
    for distro in $DISTROS; do
	[ -d "$part/$distro" ] || fail "Missing directory '$part/$distro'"
    done
}

[ $# = 2 ] || usage

PARTS="kmod_kmp_bin kmod_kmp_src kmod_kmp_debuginfo"
case "$1" in
    planet9)
	DISTROS="RHEL7.6 RHEL8.0 SLES15SP1"
	DOXBOXPARTS="BIN=>SF-121791-LS  SRC=>SF-121792-LS DBGINFO=>SF-121793-LS"
	;;
    neptune)
	DISTROS="RHEL7.7 RHEL8.1 SLES15SP1"
	DOXBOXPARTS="BIN=>SF-122714-LS  SRC=>SF-122715-LS DBGINFO=>SF-122716-LS"
	;;
    *)
	usage
	;;
esac  
REL=$2

[ -d "$REL" ] || fail "'${REL}' is not a directory"
cd "$REL"
for part in $PARTS; do
    checkdistros "$part"
    cd "$part"
    zip -r "../${part}.zip" *
    find -type f | sort | xargs md5sum > ../${part}.md5sum
    cd ..
done

tree "../${REL}"

echo "Compare to previous releases example: sdiff X0{7,9}/*_bin.md5sum "

echo "${DOXBOXPARTS}"
echo 
echo OK
